@@ -135,13 +135,18 @@ Sendable = Union[Response, str, bytes, None]
 Responsive = Callable[[Request], Sendable]
 
 
+def add_sign(_):
+    """Do nothing."""
+    return
+
+
 class Interface:
     """The HTTP server handler. It produces Responses to send."""
 
     def __init__(self, get_or_method: Union[dict[Method, Responsive], Responsive] = None,
-                 generic: Responsive = Response(405), fallback: Responsive = Response(500),
-                 pre: list[Callable[[Request], Union[Request, Sendable]]] = None,
-                 post: list[Callable[[Request, Response], Sendable]] = None,
+                 generic: Responsive = None, fallback: Responsive = None,
+                 fpre: list[Callable[[Request], Union[Request, Sendable]]] = None,
+                 fpost: list[Callable[[Request, Response], Sendable]] = None,
                  desc: str = None, strict: bool = False):
         r"""
         :param get_or_method: a method-responsive-style dict. If a Responsive object is given, it will be GET handler
@@ -150,23 +155,24 @@ class Interface:
             it`s return value will be the final response
         :param strict: whether the interface will catch extra path in interfaces and return a 404 response
         :param desc: description about the interface. It will show instead of default message when calling __repr__
-        :param pre: things to do before processing request, it will be sent as final response
+        :param fpre: things to do before processing request, it will be sent as final response
             if a Response object is returned
-        :param post: things to do after the function processed request
+        :param fpost: things to do after the function processed request
         """
-        if get_or_method is None:
-            self.methods = {}
-        elif isinstance(get_or_method, dict):
-            self.methods = get_or_method
-        else:
-            # assume it is a Responsive object
-            self.methods = {'GET': get_or_method}
+        if isinstance(get_or_method, dict):
+            for m in get_or_method:
+                setattr(self, m.lower(), get_or_method[m])
+        elif isinstance(get_or_method, Callable):
+            setattr(self, 'get', get_or_method)
+
+        if generic is not None:
+            self.generic = generic
+        if fallback is not None:
+            self.fallback = fallback
+
         self.default_methods = {'HEAD': self.head_, 'OPTIONS': self.options_}
-        self.methods: dict[Method, Responsive] = self.methods
-        self.generic = generic
-        self._fallback = fallback
-        self.pre = pre or []
-        self.post = post or []
+        self.fpre = fpre or []
+        self.fpost = fpost or []
         self.desc = desc
         self.strict = strict
 
@@ -180,62 +186,117 @@ class Interface:
         else:
             raise ValueError(f'{obj} is not responsive nor callable')
 
-    def _select_method(self, request: Request) -> Sendable:
+    def _select_method(self, request: Request) -> Responsive:
         """Return a response which is produced by specified method in request"""
         method = request.method
         if method not in Method.__args__:
             logging.warning(f'Request method {method} is invaild. Sending 400-Response instead.')
             return Response(400)  # incorrect request method
 
-        handler = self.methods.get(method)
-        if handler is not None:
-            return handler(request)
+        if self.has_method(method):
+            return getattr(self, method.lower())
+        elif method in self.default_methods:
+            return self.default_methods[method]
         else:
-            if method in self.default_methods:
-                return self.default_methods[method](request)
-            else:
-                return self.generic(request)
+            return self.generic
 
-    def fallback(self, request: Request) -> Response:
-        return Response.create_from(self._fallback(request))
+    def has_method(self, method: str):
+        return getattr(self, method.lower(), None) is not None
+
+    def find_methods(self):
+        return tuple(m for m in Method.__args__ if self.has_method(m))
 
     def process(self, request: Request) -> Response:
         """
         Let target function process the request and return the result\n
+        *Note*: You **MUST NOT** override this method\n
         :param request: the request to process
         """
         if self.strict and (request.path != ''):
             logging.warning(f'Request path {request.path} is out of root directory. Sending 404-Response instead')
             return Response(404)
 
-        for pre in self.pre:
+        for pre in self.fpre:
             res = pre(request)
             if isinstance(res, Request):
                 request = res
             elif isinstance(res, Sendable):
                 return Response.create_from(res)
 
-        resp = Response.create_from(self._select_method(request))
-        for pst in self.post:
+        handler = self._select_method(request)
+        try:
+            resp = Response.create_from(handler(request))
+        except Exception:
+            logging.warning(f'Exception detected during processing {request} with {self}. '
+                            f'Using fallback.', exc_info = True)
+            resp = Response.create_from(self.fallback(request))
+        for pst in self.fpost:
             resp = Response.create_from(pst(request, resp))
         return resp
 
     def options_(self, request: Request) -> Response:
         """Default "OPTIONS" method implementation"""
-        resp = Response(header = {'Allow': ','.join(self.methods.keys())})
+        resp = Response(header = {'Allow': ','.join(self.find_methods())})
         if request.header.get('Origin'):
             resp.header.update({'Access-Control-Allow-Origin': request.header['Origin']})
         return resp
 
     def head_(self, request: Request) -> Response:
         """Default "HEAD" method implementation"""
-        if 'GET' not in self.methods and self.generic == Response(405):
-            return Response(405)
+        if not self.has_method('GET'):
+            return self.generic(request)
         request.method = 'GET'
         resp = Response.create_from(self.process(request))
         resp.content = b''
         return resp
 
+    # register signatures for potential method definitions
+    # You may override these functions
+    @add_sign
+    def get(self, request: Request) -> Sendable:
+        ...
+
+    @add_sign
+    def post(self, request: Request) -> Sendable:
+        ...
+
+    @add_sign
+    def put(self, request: Request) -> Sendable:
+        ...
+
+    @add_sign
+    def delete(self, request: Request) -> Sendable:
+        ...
+
+    @add_sign
+    def head(self, request: Request) -> Sendable:
+        ...
+
+    @add_sign
+    def connect(self, request: Request) -> Sendable:
+        ...
+
+    @add_sign
+    def trace(self, request: Request) -> Sendable:
+        ...
+
+    @add_sign
+    def options(self, request: Request) -> Sendable:
+        ...
+
+    @add_sign
+    def patch(self, request: Request) -> Sendable:
+        ...
+
+    def generic(self, request: Request) -> Sendable:
+        logging.warning(f'request method {request.method} is not defined in {self}, sending 405-response.')
+        return Response(405)
+
+    @staticmethod
+    def fallback(request: Request) -> Response:
+        logging.warning(f'Sending default fallback response for {request}')
+        return Response(500)
+
     def __call__(self, *args, **kwargs):
         return self.process(*args, **kwargs)
 
@@ -248,13 +309,9 @@ class Interface:
         template = self.__class__.__name__ + '[{}]'
         if self.desc:
             return template.format(self.desc)
-        if not self.methods:
+        if not self.find_methods():
             return template.format(name(self.generic))
-
-        method_map = []
-        for method, func in self.methods.items():
-            method_map.append(method.upper() + ':' + name(func))
-        return template.format('|'.join(method_map))
+        return template.format('|'.join(self.find_methods()))
 
 
 class Node(Interface):
@@ -268,7 +325,7 @@ class Node(Interface):
         :param interface_callback: the function to call whenever getting mapping in order to modify mapping dynamically
         :param default: the default root interface
         """
-        super().__init__(generic = self._process, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.map_static: dict[str, Interface] = interface_map or {}
         self.map_callback = interface_callback
         self.bind('/', default)
@@ -295,7 +352,7 @@ class Node(Interface):
         else:
             raise ValueError(f'Path "{req_path}" is not a valid path')
 
-    def _process(self, request: Request) -> Sendable:
+    def generic(self, request: Request) -> Sendable:
         target, request = self.select_target(request)
         self.last_call = repr(target)
         return target(request)
@@ -320,8 +377,9 @@ class Node(Interface):
             pattern = pattern.removesuffix('/')
         if isinstance(interface_or_method, Interface) or hasattr(interface_or_method, '__call__'):
             # called as a normal function
+            if self.map_static:  # avoid log event for binding default interface
+                logging.info(f'{interface_or_method} is bound on {pattern}')
             self.map_static[pattern] = interface_or_method
-            logging.info(f'{interface_or_method} is bound on {pattern}')
             return
 
         def decorator(func):  # called as a decorator
